@@ -2,12 +2,12 @@ package DBIx::DBO::Table;
 
 use strict;
 use warnings;
-use DBIx::DBO::Common;
 use Carp 'croak';
-our @ISA = qw(DBIx::DBO::Common);
 
 use overload '**' => \&column, fallback => 1;
 
+sub _row_class { $_[0]{DBO}->_row_class }
+
 =head1 NAME
 
 DBIx::DBO::Table - An OO interface to SQL queries and results.  Encapsulates a table in an object.
@@ -54,7 +54,6 @@ sub new {
     my $proto = shift;
     UNIVERSAL::isa($_[0], 'DBIx::DBO') or croak 'Invalid DBO Object';
     my $class = ref($proto) || $proto;
-    $class = $class->_set_dbd_inheritance($_[0]{dbd});
     $class->_init(@_);
 }
 
@@ -81,7 +80,7 @@ sub _table_alias {
 
 sub _quoted_name {
     my $me = shift;
-    defined $me->{_quoted_name} ? $me->{_quoted_name} : ($me->{_quoted_name} = $me->_qi(@$me{qw(Schema Name)}));
+    defined $me->{_quoted_name} ? $me->{_quoted_name} : ($me->{_quoted_name} = $me->{DBO}{dbd_class}->_qi($me, @$me{qw(Schema Name)}));
 }
 
 =head3 C<columns>
@@ -106,7 +105,7 @@ The C<**> method is a shortcut for the C<column> method.
 
 sub column {
     my($me, $col) = @_;
-    croak 'Invalid column '.$me->_qi($col).' in table '.$me->_quoted_name
+    croak 'Invalid column '.$me->{DBO}{dbd_class}->_qi($me, $col).' in table '.$me->_quoted_name
         unless exists $me->{Column_Idx}{$col};
     $me->{Column}{$col} ||= bless [$me, $col], 'DBIx::DBO::Column';
 }
@@ -134,7 +133,8 @@ sub row {
 Fetch the first matching row from the table returning it as a L<Row|DBIx::DBO::Row> object.
 
 The C<%where> is a hash of field/value pairs.
-The value can be a SCALAR reference, which will be used without quoting, or an ARRAY reference for multiple C<IN> values.
+The value can be a simple SCALAR or C<undef> for C<NULL>
+It can also be a SCALAR reference, which will be used without quoting, or an ARRAY reference for multiple C<IN> values.
 
   $someone = $table->fetch_row(age => 21, join_date => \'CURDATE()', end_date => undef);
   $a_child = $table->fetch_row(name => \'NOT NULL', age => [5 .. 15]);
@@ -155,13 +155,12 @@ Fetch the first matching row from the table returning the value in one column.
 =cut
 
 sub fetch_value {
-    my ($me, $col) = splice @_, 0, 2;
-    $col = $me->_parse_col($col);
-    my $sql = 'SELECT '.$me->_qi($col->[1]).' FROM '.$me->_quoted_name;
+    my($me, $col) = splice @_, 0, 2;
+    $col = $me->{DBO}{dbd_class}->_parse_col($me, $col);
+    my $sql = 'SELECT '.$me->{DBO}{dbd_class}->_qi($me, $col->[1]).' FROM '.$me->_quoted_name;
     my @bind;
-    $sql .= ' WHERE '.$_ if $_ = $me->_build_quick_where(\@bind, @_);
-    $me->_sql($sql, @bind);
-    my $ref = $me->rdbh->selectrow_arrayref($sql, undef, @bind);
+    $sql .= ' WHERE '.$_ if $_ = $me->{DBO}{dbd_class}->_build_quick_where($me, \@bind, @_);
+    my $ref = $me->{DBO}{dbd_class}->_selectrow_arrayref($me, $sql, undef, @bind);
     return $ref && $ref->[0];
 }
 
@@ -177,9 +176,8 @@ sub fetch_hash {
     my $me = shift;
     my $sql = 'SELECT * FROM '.$me->_quoted_name;
     my @bind;
-    $sql .= ' WHERE '.$_ if $_ = $me->_build_quick_where(\@bind, @_);
-    $me->_sql($sql, @bind);
-    $me->rdbh->selectrow_hashref($sql, undef, @bind);
+    $sql .= ' WHERE '.$_ if $_ = $me->{DBO}{dbd_class}->_build_quick_where($me, \@bind, @_);
+    $me->{DBO}{dbd_class}->_selectrow_hashref($me, $sql, undef, @bind);
 }
 
 =head3 C<fetch_column>
@@ -191,12 +189,12 @@ Fetch all matching rows from the table returning an arrayref of the values in on
 =cut
 
 sub fetch_column {
-    my ($me, $col) = splice @_, 0, 2;
-    $col = $me->_parse_col($col);
-    my $sql = 'SELECT '.$me->_qi($col->[1]).' FROM '.$me->_quoted_name;
+    my($me, $col) = splice @_, 0, 2;
+    $col = $me->{DBO}{dbd_class}->_parse_col($me, $col);
+    my $sql = 'SELECT '.$me->{DBO}{dbd_class}->_qi($me, $col->[1]).' FROM '.$me->_quoted_name;
     my @bind;
-    $sql .= ' WHERE '.$_ if $_ = $me->_build_quick_where(\@bind, @_);
-    $me->_sql($sql, @bind);
+    $sql .= ' WHERE '.$_ if $_ = $me->{DBO}{dbd_class}->_build_quick_where($me, \@bind, @_);
+    $me->{DBO}{dbd_class}->_sql($me, $sql, @bind);
     return $me->rdbh->selectcol_arrayref($sql, undef, @bind);
 }
 
@@ -220,28 +218,20 @@ sub insert {
     my @bind;
     my %remove_duplicates;
     while (@_) {
-        my @val = $me->_parse_val(pop);
-        my $col = $me->_build_col($me->_parse_col(pop));
+        my @val = $me->{DBO}{dbd_class}->_parse_val($me, pop);
+        my $col = $me->{DBO}{dbd_class}->_build_col($me, $me->{DBO}{dbd_class}->_parse_col($me, pop));
         next if $remove_duplicates{$col}++;
         push @cols, $col;
-        push @vals, $me->_build_val(\@bind, @val);
+        push @vals, $me->{DBO}{dbd_class}->_build_val($me, \@bind, @val);
     }
     my $sql = 'INSERT INTO '.$me->_quoted_name.' ('.join(', ', @cols).') VALUES ('.join(', ', @vals).')';
-    $me->_sql($sql, @bind);
+    $me->{DBO}{dbd_class}->_sql($me, $sql, @bind);
     my $sth = $me->dbh->prepare($sql) or return undef;
     my $rv = $sth->execute(@bind) or return undef;
-    $me->{LastInsertID} = $me->_save_last_insert_id($sth);
+    $me->{LastInsertID} = $me->{DBO}{dbd_class}->_save_last_insert_id($me, $sth);
     return $rv;
 }
 
-sub _save_last_insert_id {
-    # Should be provided in a DBD specific method
-    # It is called after insert and must return the autogenerated ID
-
-    # my($me, $sth) = @_;
-    # return $sth->{Database}->last_insert_id(undef, @$me{qw(Schema Name)}, undef);
-}
-
 =head3 C<last_insert_id>
 
   $table->insert(name => 'Quentin');
@@ -285,70 +275,16 @@ sub bulk_insert {
     my @cols;
     if (defined $opt{columns}) {
         @cols = map $me->column($_), @{$opt{columns}};
-        $sql .= ' ('.join(', ', map $me->_build_col($_), @cols).')';
+        $sql .= ' ('.join(', ', map $me->{DBO}{dbd_class}->_build_col($me, $_), @cols).')';
         @cols = map $_->[1], @cols;
     } else {
         @cols = @{$me->{Columns}};
     }
     $sql .= ' VALUES ';
 
-    $me->_do_bulk_insert($sql, \@cols, %opt);
+    $me->{DBO}{dbd_class}->_bulk_insert($me, $sql, \@cols, %opt);
 }
 
-sub _fast_bulk_insert {
-    my($me, $sql, $cols, %opt) = @_;
-
-    my @vals;
-    my @bind;
-    if (ref $opt{rows}[0] eq 'ARRAY') {
-        for my $row (@{$opt{rows}}) {
-            push @vals, '('.join(', ', map $me->_build_val(\@bind, $me->_parse_val($_)), @$row).')';
-        }
-    } else {
-        for my $row (@{$opt{rows}}) {
-            push @vals, '('.join(', ', map $me->_build_val(\@bind, $me->_parse_val($_)), @$row{@$cols}).')';
-        }
-    }
-
-    $sql .= join(",\n", @vals);
-    $me->do($sql, undef, @bind);
-}
-
-sub _safe_bulk_insert {
-    my($me, $sql, $cols, %opt) = @_;
-
-    # TODO: Wrap in a transaction
-    my $rv;
-    my $sth;
-    my $prev_vals = '';
-    if (ref $opt{rows}[0] eq 'ARRAY') {
-        for my $row (@{$opt{rows}}) {
-            my @bind;
-            my $vals = '('.join(', ', map $me->_build_val(\@bind, $me->_parse_val($_)), @$row).')';
-            $me->_sql($sql.$vals, @bind);
-            if ($prev_vals ne $vals) {
-                $sth = $me->dbh->prepare($sql.$vals) or return undef;
-                $prev_vals = $vals;
-            }
-            $rv += $sth->execute(@bind) or return undef;
-        }
-    } else {
-        for my $row (@{$opt{rows}}) {
-            my @bind;
-            my $vals = '('.join(', ', map $me->_build_val(\@bind, $me->_parse_val($_)), @$row{@$cols}).')';
-            $me->_sql($sql.$vals, @bind);
-            if ($prev_vals ne $vals) {
-                $sth = $me->dbh->prepare($sql.$vals) or return undef;
-                $prev_vals = $vals;
-            }
-            $rv += $sth->execute(@bind) or return undef;
-        }
-    }
-
-    return $rv || '0E0';
-}
-*_do_bulk_insert = \&_safe_bulk_insert;
-
 =head3 C<delete>
 
   $table->delete(name => 'Richard', age => 103);
@@ -361,8 +297,8 @@ sub delete {
     my $me = shift;
     my $sql = 'DELETE FROM '.$me->_quoted_name;
     my @bind;
-    $sql .= ' WHERE '.$_ if $_ = $me->_build_quick_where(\@bind, @_);
-    $me->do($sql, undef, @bind);
+    $sql .= ' WHERE '.$_ if $_ = $me->{DBO}{dbd_class}->_build_quick_where($me, \@bind, @_);
+    $me->{DBO}{dbd_class}->_do($me, $sql, undef, @bind);
 }
 
 =head3 C<truncate>
@@ -375,13 +311,17 @@ Truncate the table.  Returns true on success or C<undef> on failure.
 
 sub truncate {
     my $me = shift;
-    $me->do('TRUNCATE TABLE '.$me->_quoted_name);
+    $me->{DBO}{dbd_class}->_do($me, 'TRUNCATE TABLE '.$me->_quoted_name);
 }
 
 =head2 Common Methods
 
 These methods are accessible from all DBIx::DBO* objects.
 
+=head3 C<dbo>
+
+The C<DBO> object.
+
 =head3 C<dbh>
 
 The I<read-write> C<DBI> handle.
@@ -390,13 +330,11 @@ The I<read-write> C<DBI> handle.
 
 The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I<read-write> C<DBI> handle.
 
-=head3 C<do>
+=cut
 
-  $table->do($statement)         or die $table->dbh->errstr;
-  $table->do($statement, \%attr) or die $table->dbh->errstr;
-  $table->do($statement, \%attr, @bind_values) or die ...
-
-This provides access to L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
+sub dbo { $_[0]{DBO} }
+sub dbh { $_[0]{DBO}->dbh }
+sub rdbh { $_[0]{DBO}->rdbh }
 
 =head3 C<config>
 
@@ -412,8 +350,8 @@ See L<DBIx::DBO/Available_config_options>.
 sub config {
     my $me = shift;
     my $opt = shift;
-    return $me->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
-    return defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
+    return $me->{DBO}{dbd_class}->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
+    $me->{DBO}{dbd_class}->_get_config($opt, $me->{Config} ||= {}, $me->{DBO}{Config}, \%DBIx::DBO::Config);
 }
 
 sub DESTROY {
@@ -424,16 +362,6 @@ sub DESTROY {
 
 __END__
 
-=head1 TODO LIST
-
-=over 4
-
-=item *
-
-Add a multi_insert method for extended INSERTs.
-
-=back
-
 =head1 SEE ALSO
 
 L<DBIx::DBO>
